"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
Object.defineProperty(exports, "__esModule", { value: true });
var patchedSym = Symbol('axiosBetterStacktrace.patched');
var isAxiosError = function (error) {
    return error instanceof Error && error.isAxiosError !== undefined;
};
var axiosMethods = [
    'request',
    'get',
    'delete',
    'head',
    'options',
    'post',
    'put',
    'patch',
];
// inherits original axios handler generics https://github.com/axios/axios/blob/79979d9214601478b67a330fc144cad25c59f3c7/index.d.ts#L139-L146
var axiosBetterStacktraceHandler = function (params, topmostError) {
    // extends request config with topmostError, so it could be used inside an interceptor to enhance original error
    switch (params.method) {
        case 'request': {
            var originalHandler = params.originalHandler, config = params.args[0];
            return originalHandler(__assign(__assign({}, (config || {})), { topmostError: topmostError }));
        }
        case 'get':
        case 'delete':
        case 'head':
        case 'options': {
            var originalHandler = params.originalHandler, _a = params.args, url = _a[0], config = _a[1];
            return originalHandler(url, __assign(__assign({}, (config || {})), { topmostError: topmostError }));
        }
        case 'post':
        case 'put':
        case 'patch': {
            var originalHandler = params.originalHandler, _b = params.args, url = _b[0], data = _b[1], config = _b[2];
            return originalHandler(url, data, __assign(__assign({}, (config || {})), { topmostError: topmostError }));
        }
    }
};
var axiosBetterStacktrace = function (axiosInstance, opts) {
    if (opts === void 0) { opts = {}; }
    var _a = opts.errorMsg, errorMsg = _a === void 0 ? 'Axios Better Stacktrace' : _a;
    // do nothing if input does not look like an axios instance
    if (!axiosInstance || !axiosMethods.some(function (method) { return axiosInstance.hasOwnProperty(method); })) {
        return;
    }
    // avoid potential memory leaks if axios instance already patched
    if (axiosInstance[patchedSym]) {
        return;
    }
    var originalHandlers = {
        request: axiosInstance['request'],
        get: axiosInstance['get'],
        delete: axiosInstance['delete'],
        head: axiosInstance['head'],
        options: axiosInstance['options'],
        post: axiosInstance['post'],
        put: axiosInstance['put'],
        patch: axiosInstance['patch'],
    };
    // enhance original error with a topmostError stack trace
    axiosInstance.interceptors.response.use(undefined, function (error) {
        if (isAxiosError(error) && error.config && error.config.topmostError instanceof Error) {
            error.originalStack = error.stack;
            error.stack = error.stack + "\n" + error.config.topmostError.stack;
            // remove topmostError to not clutter config and expose it to other interceptors down the chain
            delete error.config.topmostError;
            throw error;
        }
        throw error;
    });
    axiosMethods.forEach(function (method) {
        if (method in axiosInstance) {
            switch (method) {
                case 'request': {
                    var originalHandler_1 = axiosInstance[method];
                    axiosInstance[method] = function axiosBetterStacktraceMethodProxy(config) {
                        return axiosBetterStacktraceHandler({
                            method: method,
                            originalHandler: originalHandler_1,
                            args: [config],
                        }, new Error(errorMsg));
                    };
                    break;
                }
                case 'get':
                case 'delete':
                case 'head':
                case 'options': {
                    var originalHandler_2 = axiosInstance[method];
                    axiosInstance[method] = function axiosBetterStacktraceMethodProxy(url, config) {
                        return axiosBetterStacktraceHandler({
                            method: method,
                            originalHandler: originalHandler_2,
                            args: [url, config],
                        }, new Error(errorMsg));
                    };
                    break;
                }
                case 'post':
                case 'put':
                case 'patch': {
                    var originalHandler_3 = axiosInstance[method];
                    axiosInstance[method] = function axiosBetterStacktraceMethodProxy(url, data, config) {
                        return axiosBetterStacktraceHandler({
                            method: method,
                            originalHandler: originalHandler_3,
                            args: [url, data, config],
                        }, new Error(errorMsg));
                    };
                    break;
                }
            }
            if (!axiosInstance[patchedSym]) {
                axiosInstance[patchedSym] = true;
            }
        }
    });
    // ensure consumer of the plugin can restore original handlers
    var restoreOriginalHandlers = function () {
        Object.assign(axiosInstance, originalHandlers);
    };
    return restoreOriginalHandlers;
};
exports.default = axiosBetterStacktrace;
//# sourceMappingURL=axiosBetterStacktrace.js.map